
import java.util.*;
import com.nec.tdd.tools.dbMapper.*;
import com.nec.tdd.tools.logger.Logger;


public class Test {

   DBInterface mapper; // mapper used for this test

   Test (String mapperFile) {

      // create the mapper to be used for this test
      try {
         DBModule dbm = DBModule.init (mapperFile);
         mapper = dbm.createDefaultMapper ("default");
         Logger.debug (mapper.toString());
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
         System.exit(0);
      }
      this.mapper = mapper;
   }

   public static void main (String[] args) {
      try {
         String mapperFile = args[0];
         Test testSuite = new Test(mapperFile);
         testSuite.test();
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
      }
   }

   void test() throws Exception {
      AttrValMap aValMap = null;
      HashMap    hValMap = null;

      // Delete older entries
      mapper.deleteAll (User.class);

      // Create a User object and persist it to database
      User usr1 = new User ("Charles", "Smith", 13452,
                            'M', 170.34, "csmith@mailcity.com",
                            true, new Integer(55000), new Short((short)15));
      mapper.create (usr1);
      Logger.debug ("create(Object) :: Object " + usr1
                    + " is successfully created.");

      // Now delete the user created above using primay key
      UserPK usr1PK = new UserPK("Charles", "Smith", 13452);
      mapper.deleteByPrimaryKey (usr1PK, User.class);
      Logger.debug ("delete(Object) :: Object " + usr1
                    + " is successfully deleted.");

      // Create usr1 again. Delete all the User objects with last name "Smith"
      // i.e. (deletes usr1 from database)
      mapper.create (usr1);
      Logger.debug ("create(Object) :: Object " + usr1
                   + " is successfully created.");
      aValMap = new AttrValMap();
      aValMap.put ("lastName", "Smith");
      mapper.deleteByAttributes (aValMap, User.class);
      Logger.debug ("deleteByAttributes(aval,cls) :: Object " + usr1
                    + " is successfully deleted.");

      // Create the usr1 again. Update usr1 attributes and update new changes
      // to database
      mapper.create (usr1);
      Logger.debug ("create(Object) :: Object " + usr1
                   + " is successfully created.");
      usr1.setDob (new Short((short) 18));
      usr1.setEmail (null);
      usr1.setIncome (null);
      mapper.update(usr1);
      Logger.debug ("update(Object) :: Object " + usr1
                   + " is successfully updated.");

      // Update selected attributes (using AttrValMap)
      // of usr1 and update these new changes to database
      aValMap = new AttrValMap();
      aValMap.put ("dob", new Short((short) 12));
      aValMap.put ("income", new Integer(67759));
      aValMap.put ("email", null);
      mapper.update(usr1, aValMap, true);
      Logger.debug ("update(Object,aval,bool) :: Object " + usr1
                   + " is successfully updated.");


      // Update selected attributes (using HashMap) of usr1 and update
      // these new changes to database
      hValMap = new HashMap(1);
      hValMap.put ("income", new Integer(23234));
      hValMap.put ("dob", new Short((short)30));
      hValMap.put ("email", "smith@hotmail.com");
      mapper.update(usr1, hValMap, true);
      Logger.debug ("update(Object,hval,bool) :: Object " + usr1
                  + " is successfully updated.");


      // Create few more users
      User usr2 = new User ("Goldy", "Smith", 786, 'F', -1,
                            null, false, new Integer(1000000),
                            new Short((short)24));
      mapper.create (usr2);
      User usr3 = new User ("Kate", "Winslet", 1234, 'F', 168.23,
                           "katie@hollywood.com", true, new Integer(999699),
                           new Short((short)12));
      mapper.create (usr3);
      User usr4 = new User ("Princess", "Diana", 666, 'F', 178,
                            "diana@celebs.com", false, new Integer(666978),
                            new Short((short)6));
      mapper.create (usr4);


      // Locate an previously created user in database using primary key
      UserPK usr2PK = new UserPK ("Goldy", "Smith", 786);
      User usr2dup = (User) mapper.findByPrimaryKey (
                                    usr2PK, User.class); // Should return usr2
      Logger.debug ("findByPrimaryKey(pk,class) :: Object " + usr2dup
                    + " is successfully found.");


      // Locate an previously created user in database using key attribues
      aValMap = new AttrValMap();
      aValMap.put ("firstName", "Princess");
      aValMap.put ("lastName", "Diana");
      aValMap.put ("pin", 666);
      Collection c1 = mapper.findByAttributes (aValMap, User.class);
      // c1 should contain only usr4
      Logger.debug ("findByAttributes(aval,class) :: Collection "
                  + c1.toString() + " is successfully found.");


      // Find all the alive females from database
      aValMap = new AttrValMap();
      aValMap.put ("sex", 'F');
      aValMap.put ("alive", true);
      Collection c2 = mapper.findPrimaryKeysByAttributes (aValMap,
                                                               User.class);
      Logger.debug ("findByAttributes(aval,class) :: Collection "
                    + c2.toString() + " is successfully found.");
      // Collection c2 should only contain primary object pointing to usr3.
      // Use the primary key to delete this entry from database
      Object akey = c2.iterator().next();
      mapper.deleteByPrimaryKey (akey, User.class);
      Logger.debug ("deleteByPrimaryKey(key,class) :: Object " + akey
                    + " is successfully deleted.");


      // Find User objects based on a custom SQL query
      Collection c3 = mapper.findByQuery (
                     "SELECT * from demo_user where lastName='Smith'",
                     User.class);
      Logger.debug ("findByQuery(query,class) :: Collection " + c3.toString()
                   + " is successfully found.");

      // Find all the User objects stored in demo_user table
      Collection c4 = mapper.findAll (User.class);
      Logger.debug ("findAll(class) :: Collection " + c4.toString()
                    + " is successfully found.");

      // Find a primary key object based on non-key attribute sex.
      // (find all men from database)
      aValMap = new AttrValMap();
      aValMap.put ("sex", 'M');
      Collection c5 = mapper.findPrimaryKeysByAttributes (
                                                   aValMap, User.class);
      Logger.debug ("findPrimaryKeysByAttributes(aval,class) :: Collection "
                    + c5.toString() + " is successfully found.");


      // Find primary key objects based on a custom SQL query
      Collection c6 = mapper.findPrimaryKeysByQuery (
          "SELECT pin, lastName, firstName FROM demo_user WHERE lastName"
          + " like 'Sm%' ORDER by firstName, lastName, pin",
          User.class);
      Logger.debug ("findPrimaryKeysByQuery(query,class) :: Collection "
                   + c6.toString() + " is successfully found.");


      // Find primary key of all the User objects stored in
      // demo_user table
      Collection c7 = mapper.findAllPrimaryKeys (User.class);
      Logger.debug ("findAllPrimaryKeys(class) :: Collection "
                   + c7.toString() + " is successfully found.");

      // Add usr3 ("Kate Winslet") back.
      mapper.create (usr3);
      Collection c8 = mapper.findAll (User.class);
      Logger.debug ("findAll(class) :: Collection " + c8.toString()
                    + " is successfully found.");
   }

}

