import java.util.*;
import com.nec.tdd.tools.dbMapper.*;
import com.nec.tdd.tools.logger.Logger;


public class Test {

   DBInterface mapper; // mapper used for this test

   Test (String mapperFile) {

      // create the mapper to be used for this test
      try {
         DBModule dbm = DBModule.init (mapperFile);
         mapper = dbm.createDefaultMapper ("default");
         Logger.debug (mapper.toString());
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
         System.exit(0);
      }
      this.mapper = mapper;
   }

   public static void main (String[] args) {
      try {
         String mapperFile = args[0];
         Test testSuite = new Test(mapperFile);
         testSuite.test();
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
      }
   }

   void test() throws Exception {
      // Delete older entries
      mapper.deleteAll (Person.class);

      // Instantiate all House objects owned by "Charles Smith"
      Address r1 = new Address (1, "Charles Smith", 101, "12056 Greywing Sq",
                               "Reston", "VA", 20191,
                               new GeoLoc(1, (float)34.5, (float)-23.6));
      House   h1 = new House ("Charles Smith", 101, 1024, 5, r1);
      Address r2 = new Address (2, "Charles Smith", 102, "13452 Farmcrest Ct",
                               "Herndon", "VA", 20171,
                               new GeoLoc(1, (float)34.45, (float)-23.61));
      House   h2 = new House ("Charles Smith", 102, 1025, 1000000, r2);

      // Instantiate all the vehicles owned by "Charles Smith"
      Vehicle v1 = new Vehicle ("Charles Smith", 234567, "Toyota Camary", null);
      ArrayList parts1 = new ArrayList();
      parts1.add (new Part(874687, "steering",
                           "steer the vehicle in desired direction."));
      parts1.add (new Part(874687, "tyres", null));
      Vehicle v2 = new Vehicle ("Charles Smith", 874687, "Ford", parts1);

      // Instantiate the "Charles Smith" Person object with above
      // houses and vehicles
      House[] houses1 = new House[] {h1,h2};
      Vector  vehicles1 = new Vector(2);
      vehicles1.add (v1);
      vehicles1.add (v2);
      Person p1 = new Person("Charles Smith", houses1, vehicles1);

      // Persist the entire Person object containment tree (including
      // vehicles, houses, address, locations, parts) in database
      mapper.createTree (p1);
      Logger.debug ("create(Object) :: Object " + p1
                   + " is successfully created.");
      Logger.debug ("-------------------------------------------------------");

      // Find the persistent "Charles Smith" Person object in database
      // Load the entire Person object containment tree from database
      // in another Person object
      Person p1Dup = (Person) mapper.findByPrimaryKey ("Charles Smith",
                                                         Person.class, 9999);
      // Compare p1 and p1Dup object containment tree in the log file,
      // field by field.  The fields should have same value.
      Logger.debug ("findByPrimaryKey() :: Object " + p1Dup
                   + " is successfully executed.");
   }

}

